from typing import Union, Any, Optional, Type, TYPE_CHECKING, overload
from typing_extensions import override
import random
import json

from nonebot.message import handle_event
from nonebot.adapters import Bot as BaseBot

from .event import Event
from .message import (
    Message,
    MessageSegment,
)

if TYPE_CHECKING:
    from .adapter import Adapter


class Bot(BaseBot):
    adapter: "Adapter"

    @override
    def __init__(self, adapter: "Adapter", self_id: str):
        super().__init__(adapter, self_id)

        # Bot 鉴权信息
        self._access_token: Optional[str] = None
        self._expires_in: Optional[int] = None

    @override
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        **kwargs,
    ) -> Any:
        """发送消息"""
        if isinstance(message, str):
            message = Message(MessageSegment.text(message))
        elif isinstance(message, MessageSegment):
            message = Message(message)
        elif not isinstance(message, Message):
            raise ValueError("message type error")

        msg = message.extract_plain_text()

        if not msg:
            raise ValueError("Empty message")

        if group_id := getattr(event, "group_id", None):
            await self.send_group_msg(group_id, msg)
        elif user_id := getattr(event, "user_id", None):
            await self.send_private_msg(user_id, msg)
        else:
            raise ValueError(
                "Unknown recipient: neither group_id nor user_id found in event"
            )

    async def handle_event(self, event: Type[Event]):
        """处理事件"""
        if event.get_user_id() == self.self_id:
            return

        atToInfo: list[dict] = event.ext.get("atToInfo", [])
        if any(item.get("uid", "") == self.self_id for item in atToInfo):
            event.tome = True

        await handle_event(self, event)

    async def send_private_msg(self, user_id: str, msg: str) -> None:
        """发送消息"""
        ws = self.adapter.connections[self.self_id]
        await ws.send_text(
            json.dumps(
                {
                    "chatType": "singleChat",
                    "ext": {},
                    "from": self.self_id,
                    "id": random.randint(int(10e12), int(10e15)),
                    "msg": msg,
                    "to": user_id,
                    "type": "txt",
                }
            )
        )

    async def send_group_msg(self, group_id: str, msg: str) -> None:
        """发送消息"""
        ws = self.adapter.connections[self.self_id]
        await ws.send_text(
            json.dumps(
                {
                    "group": "groupchat",
                    "ext": {},
                    "from": self.self_id,
                    "id": random.randint(int(10e12), int(10e15)),
                    "msg": msg,
                    "to": group_id,
                    "type": "txt",
                }
            )
        )

    @overload
    async def send_msg(self, user_id: str, msg: str) -> None: ...

    @overload
    async def send_msg(self, group_id: str, msg: str) -> None: ...

    async def send_msg(self, **kwargs) -> None:
        """发送消息"""
        if "user_id" in kwargs:
            await self.send_private_msg(kwargs["user_id"], kwargs["msg"])
        elif "group_id" in kwargs:
            await self.send_group_msg(kwargs["group_id"], kwargs["msg"])
        else:
            raise ValueError("Unknown message type")



from typing import Any, Optional, cast, Literal
from pydantic import (
    Field,
    ConfigDict,
    model_validator,
    HttpUrl,
)
from typing_extensions import override
import datetime

from nonebot.adapters import Event as BaseEvent
from nonebot.compat import model_dump

from .message import Message


class Event(BaseEvent):
    ext: dict = Field(default_factory=dict)
    """ 额外信息 """
    tome: bool = Field(default=False)
    """ 是否为机器人相关消息 """

    model_config = ConfigDict(extra="allow")

    @override
    def is_tome(self) -> bool:
        return self.to_me

    @override
    def get_type(self) -> Literal["message", "notice", "request", "meta_event"]:
        raise NotImplementedError

    @override
    def get_message(self) -> Optional["Message"]:
        raise NotImplementedError

    @override
    def get_event_name(self) -> str:
        return self.__class__.__name__

    @override
    def get_event_description(self) -> str:
        return str(model_dump(self))

    @override
    def get_user_id(self) -> Optional[str]:
        return None

    @override
    def get_session_id(self) -> str:
        raise NotImplementedError

    def get_event_id(self) -> str:
        raise NotImplementedError


class MetaEvent(Event):
    """元事件"""

    @override
    def get_type(self) -> Literal["meta_event"]:
        return "meta_event"


class NoticeEvent(Event):
    """通知事件"""

    @override
    def get_type(self) -> Literal["notice"]:
        return "notice"


class MessageEvent(Event):
    """消息事件"""

    message_id: str = Field(alias="id")
    """ 消息ID """
    message_type: Literal["private", "group"] = Field(alias="type")
    """ 消息类型 """
    user_id: str = Field(alias="from")
    """ 发送者ID """
    to_user_id: str = Field(alias="to")
    """ 接收者ID """
    content_type: Literal["text", "image", "voice"] = Field(alias="contentsType")
    """ 消息内容类型 """
    timestamp: int = Field(alias="time")
    """ 时间戳 """
    ext: dict = Field(default_factory=dict)
    """ 额外信息 """

    model_config = ConfigDict(extra="ignore")

    @model_validator(mode="before")
    def _(cls, values: dict[str, Any]) -> dict[str, Any]:
        """数据预处理"""
        if _type := values.get("type"):
            values["type"] = {
                "chat": "private",
                "groupchat": "group",
            }.get(_type, _type)

        values["contentsType"] = cast(str, values.get("contentsType", "")).lower()
        return values

    @property
    def time(self) -> datetime.datetime:
        return datetime.datetime.fromtimestamp(self.timestamp / 1000)

    @property
    def message(self) -> Message:
        return self.get_message()

    @property
    def group_id(self) -> Optional[str]:
        """获取群聊ID"""
        return self.get_group_id()

    @override
    def is_tome(self) -> bool:
        if self.message_type == "private":
            return True
        else:
            return getattr(self, "tome", False)

    @override
    def get_type(self) -> Literal["message"]:
        return "message"

    @override
    def get_message(self) -> Message:
        return Message.from_event(self)

    @override
    def get_event_description(self) -> str:
        keys = (
            "user_id",
            "to_user_id",
            "time",
            "message_type",
            "message_id",
            "message",
        )
        return str({key: getattr(self, key) for key in keys})

    @override
    def get_user_id(self) -> str:
        return str(self.user_id)

    def get_group_id(self) -> Optional[str]:
        """获取群聊ID"""
        if self.message_type == "group":
            return self.to_user_id
        return None

    @override
    def get_session_id(self) -> str:
        return f"{self.message_type}_{self.user_id}_{self.to_user_id}"

    @override
    def get_event_id(self) -> str:
        return f"{self.message_id}_{self.get_session_id()}"


class CommandEvent(Event):
    """指令事件"""

    message_id: str = Field(alias="id")
    """ 消息ID """
    user_id: str = Field(alias="from")
    """ 发送者ID """
    self_id: str = Field(alias="to")
    """ 接收者ID """
    content_type: Literal["command"] = Field(alias="contentsType")
    """ 消息内容类型 """
    timestamp: int = Field(alias="time")
    """ 时间戳 """
    action: str = Field()
    """ 指令动作 """

    model_config = ConfigDict(extra="ignore")

    @model_validator(mode="before")
    def _(cls, values: dict[str, Any]) -> dict[str, Any]:
        """数据预处理"""
        if _type := values.get("type"):
            values["type"] = {
                "chat": "private",
                "groupchat": "group",
            }.get(_type, _type)

        values["contentsType"] = cast(str, values.get("contentsType", "")).lower()
        return values

    @override
    def get_type(self) -> Literal["message"]:
        return "request"

    @override
    def get_event_name(self) -> str:
        return f"{self.__class__.__name__}.{self.action}"


class ConnectedEvent(MetaEvent):
    """连接事件"""

    type: Literal["connected"] = Field()


class ClosedEvent(MetaEvent):
    """关闭事件"""

    type: Literal["closed"] = Field()


class ErrorEvent(MetaEvent):
    """错误事件"""

    type: Literal["error"] = Field()
    data: dict = Field()


class TextMessageEvent(MessageEvent):
    """文本消息事件"""

    content_type: Literal["text"] = Field(alias="contentsType")
    """ 消息内容类型 """
    content: str = Field(alias="data")
    """ 消息内容 """


class ImageMessageEvent(MessageEvent):
    content_type: Literal["image"] = Field(alias="contentsType")
    """ 消息内容类型 """
    file_url: HttpUrl = Field(alias="url")
    """ 图片URL """
    file_name: str = Field(alias="filename")
    """ 图片名称 """
    file_length: int = Field(alias="file_length")
    """ 图片大小 """
    file_width: int = Field(alias="width")
    """ 图片宽度 """
    file_height: int = Field(alias="height")
    """ 图片高度 """

    @model_validator(mode="before")
    def _(cls, values: dict[str, Any]) -> dict[str, Any]:
        """数据预处理"""
        values["file_length"] = int(values.get("file_length", 0) or 0)
        return values


class VoiceMessageEvent(MessageEvent):
    content_type: Literal["voice"] = Field(alias="contentsType")
    """ 消息内容类型 """
    file_url: HttpUrl = Field(alias="url")
    """ 音频URL """
    file_name: str = Field(alias="filename")
    """ 音频名称 """
    file_length: int = Field(alias="file_length")
    """ 音频大小 """


EVENT_CLASSES: list[type[Event]] = [
    ConnectedEvent,
    ClosedEvent,
    ErrorEvent,
    TextMessageEvent,
    ImageMessageEvent,
    VoiceMessageEvent,
    CommandEvent,
]



from typing import Type, Union, Iterable, Optional, TYPE_CHECKING
from typing_extensions import override
from pydantic import HttpUrl

from nonebot.adapters import (
    MessageSegment as BaseMessageSegment,
    Message as BaseMessage,
)

if TYPE_CHECKING:
    from .event import MessageEvent


class MessageSegment(BaseMessageSegment["Message"]):
    """消息段"""

    @classmethod
    @override
    def get_message_class(cls) -> Type["Message"]:
        return Message

    @override
    def __str__(self) -> str:
        if self.type == "text":
            return self.data["text"]
        return ""

    @override
    def is_text(self) -> bool:
        return self.type == "text"

    @classmethod
    def text(
        cls,
        text: str,
        ext: dict = {},
    ) -> "Text":
        """文本消息段

        参数：
        - `text` 文本内容
        - `ext` 扩展字段
        """
        return Text(
            "text",
            {
                "text": text,
                "ext": ext,
            },
        )

    @classmethod
    def image(
        cls,
        file_url: Optional[HttpUrl] = None,
        file_name: Optional[str] = None,
        ext: dict = {},
    ) -> "Image":
        """图片消息段

        参数：
        - `file_url` 图片文件的网络 URL
        - `file_name` 图片文件名
        - `ext` 扩展字段
        """
        return Image(
            "image",
            {
                "file_url": file_url,
                "file_name": file_name,
                "ext": ext,
            },
        )

    @classmethod
    def voice(
        cls,
        file_url: Optional[HttpUrl] = None,
        file_name: Optional[str] = None,
        ext: dict = {},
    ) -> "Voice":
        """语音消息段

        参数：
        - `file_url` 语音文件的网络 URL
        - `file_name` 语音文件名
        - `ext` 扩展字段
        """
        return Voice(
            "voice",
            {
                "file_url": file_url,
                "file_name": file_name,
                "ext": ext,
            },
        )


class Text(MessageSegment):
    """文本 消息段"""


class Image(MessageSegment):
    """图片 消息段"""


class Voice(MessageSegment):
    """音频 消息段"""


class Message(BaseMessage[MessageSegment]):
    """消息"""

    @override
    @classmethod
    def get_segment_class(cls) -> Type[MessageSegment]:
        """获取消息段类"""
        return MessageSegment

    @override
    @staticmethod
    def _construct(msg: str) -> Iterable[MessageSegment]:
        """将文本消息构造成消息段数组"""
        yield Text("text", {"text": msg})

    @override
    def __add__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return super().__add__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @override
    def __radd__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return super().__radd__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    def extract_plain_text(self) -> str:
        """提取消息中的纯文本"""
        return "".join(seg.data["text"] for seg in self if seg.type == "text")

    @classmethod
    def from_event(cls, event: type["MessageEvent"]) -> "Message":
        """从消息事件转为消息序列"""
        if event.content_type == "text":
            return cls(
                MessageSegment.text(
                    text=getattr(event, "content", ""),
                    ext=event.ext,
                ),
            )
        elif event.content_type == "image":
            return cls(
                MessageSegment.image(
                    file_url=getattr(event, "file_url", ""),
                    ext=event.ext,
                ),
            )
        elif event.content_type == "voice":
            return cls(
                MessageSegment.voice(
                    file_url=getattr(event, "file_url", ""),
                    ext=event.ext,
                ),
            )
        else:
            raise ValueError(f"Unknown content type: {event.content_type}")



from nonebot.utils import logger_wrapper

log = logger_wrapper("ChaoXing")


def escape(s: str) -> str:
    return s.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")


def unescape(s: str) -> str:
    return s.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")



import json
from typing import Optional

from nonebot.drivers import Response
from nonebot.exception import AdapterException
from nonebot.exception import ActionFailed as BaseActionFailed
from nonebot.exception import NetworkError as BaseNetworkError
from nonebot.exception import NoLogException as BaseNoLogException
from nonebot.exception import ApiNotAvailable as BaseApiNotAvailable


class CxAdapterException(AdapterException):
    def __init__(self):
        super().__init__("ChaoXing")


class NoLogException(BaseNoLogException, CxAdapterException):
    pass


class ActionFailed(BaseActionFailed, CxAdapterException):
    def __init__(self, response: Response):
        self.response = response

        self.body: Optional[dict] = None
        if response.content:
            try:
                self.body = json.loads(response.content)
            except Exception:
                pass

    @property
    def status_code(self) -> int:
        return self.response.status_code

    @property
    def code(self) -> Optional[int]:
        return None if self.body is None else self.body.get("errcode", self.body.get("errCode", None))

    @property
    def msg(self) -> Optional[str]:
        return None if self.body is None else self.body.get("errmsg", self.body.get("errMsg", None))

    @property
    def data(self) -> Optional[dict]:
        return None if self.body is None else self.body.get("data", None)

    def __repr__(self) -> str:
        args = ("code", "msg", "data")
        return (
            f"<ActionFailed: {self.status_code}, "
            + ", ".join(f"{k}={v}" for k in args if (v := getattr(self, k)) is not None)
            + ">"
        )

    def __str__(self):
        return self.__repr__()


class NetworkError(BaseNetworkError, CxAdapterException):
    def __init__(self, msg: Optional[str] = None):
        super().__init__()
        self.msg: Optional[str] = msg
        """ 错误原因 """

    def __repr__(self):
        return f"<NetWorkError message={self.msg}>"

    def __str__(self):
        return self.__repr__()


class ApiNotAvailable(BaseApiNotAvailable, CxAdapterException):
    pass


class UnkonwnEventError(CxAdapterException):
    """ 未知事件 """

    def __init__(self, event: dict):
        self.event = event

    def __repr__(self):
        return f"<UnkonwnEvent {self.event}>"

    def __str__(self):
        return self.__repr__()



from typing_extensions import override
from pydantic import ValidationError
from typing import Any, Type
from pathlib import Path
from yarl import URL
import contextlib
import asyncio
import httpx
import json

from nonebot.exception import WebSocketClosed
from nonebot.compat import PYDANTIC_V2
from nonebot import get_plugin_config
from nonebot.utils import escape_tag
from nonebot.drivers import (
    Driver,
    Request,
    Response,
    ASGIMixin,
    WebSocket,
    HTTPClientMixin,
    HTTPServerSetup,
    WebSocketServerSetup,
)
from nonebot.adapters import Adapter as BaseAdapter

from .bot import Bot
from .event import Event, EVENT_CLASSES
from .utils import log
from .config import Config
from .exception import (
    UnkonwnEventError,
)


class Adapter(BaseAdapter):
    @override
    def __init__(self, driver: Driver, **kwargs: Any):
        super().__init__(driver, **kwargs)
        self.connections: dict[str, WebSocket] = {}
        self.access_token: dict[str, str] = {}
        self.tasks: set["asyncio.Task"] = set()
        self.cx_config: Config = get_plugin_config(Config)
        self.setup()

    @classmethod
    @override
    def get_name(cls) -> str:
        """适配器名称: `ChaoXing`"""
        return "ChaoXing"

    def setup(self) -> None:
        if not isinstance(self.driver, ASGIMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} doesn't support asgi server!"
                f"{self.get_name()} Adapter need a asgi server driver to work."
            )

        if not isinstance(self.driver, HTTPClientMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} "
                "doesn't support http client requests!"
                f"{self.get_name()} Adapter needs a HTTPClient Driver to work."
            )

        for path in ["/chaoxing", "/chaoxing/"]:
            self.setup_http_server(
                HTTPServerSetup(
                    path=URL(path),
                    method="GET",
                    name=f"{self.get_name()} GET {path}",
                    handle_func=self._handle_get,
                )
            )

        self.setup_http_server(
            HTTPServerSetup(
                path=URL("/chaoxing/login"),
                method="POST",
                name=f"{self.get_name()} Root GET",
                handle_func=self._handle_login,
            )
        )

        self.setup_websocket_server(
            WebSocketServerSetup(
                path=URL("/chaoxing/ws"),
                name=f"{self.get_name()} Root WebSocket",
                handle_func=self._handle_ws,
            )
        )

        self.driver.on_shutdown(self._shutdown)

    async def _shutdown(self) -> None:
        """关闭 Adapter"""
        for task in self.tasks:
            if not task.done():
                task.cancel()

        await asyncio.gather(
            *(asyncio.wait_for(task, timeout=10) for task in self.tasks),
            return_exceptions=True,
        )
        self.tasks.clear()

    async def _call_api(self, bot: Bot, api: str, **data: Any) -> Response:
        """调用平台 API"""
        raise NotImplementedError

    async def _handle_ws(self, ws: WebSocket) -> None:
        """处理 WebSocket 连接"""
        self_id = ws.request.url.query.get("self_id")
        access_token = ws.request.url.query.get("access_token")

        if not self_id:
            log("WARNING", "Missing Self-ID")
            await ws.close(1008, "Missing Self-ID")
            return

        if self_id in self.bots:
            log("WARNING", f"There's already a bot {self_id}, ignored")
            await ws.close(1008, "Duplicate Self-ID")
            return

        if self.cx_config.cx_token and access_token != self.cx_config.cx_token:
            log("WARNING", "Invalid access token")
            await ws.close(1008, "Invalid access token")
            return

        await ws.accept()
        bot = Bot(self, self_id)
        self.bot_connect(bot)
        self.connections[self_id] = ws

        log("INFO", f"<y>Bot {escape_tag(self_id)}</y> connected")

        try:
            while True:
                data: str = await ws.receive()
                payload: dict = json.loads(data)

                if event := self.payload_to_event(payload):
                    task = asyncio.create_task(bot.handle_event(event))
                    task.add_done_callback(self.tasks.discard)
                    self.tasks.add(task)
        except WebSocketClosed:
            log("WARNING", f"WebSocket for Bot {escape_tag(self_id)} closed by peer")
        except Exception as e:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Error while process data from websocket "
                f"for bot {escape_tag(self_id)}.</bg #f8bbd0></r>",
                e,
            )
        finally:
            with contextlib.suppress(Exception):
                await ws.close()
            self.connections.pop(self_id, None)
            self.bot_disconnect(bot)

    async def _handle_get(self, request: Request) -> Response:
        """处理 HTTP GET 请求"""
        headers = {
            "Content-Type": "text/html; charset=utf-8",
        }
        if html := getattr(self, "_html", None):
            return Response(status_code=200, content=html, headers=headers)
        else:
            with open(Path(__file__).parent / "res" / "im.html", "r") as f:
                self._html = f.read()
            return Response(status_code=200, content=self._html, headers=headers)

    async def _handle_login(self, request: Request) -> Response:
        """处理登录请求"""
        data: dict = json.loads(request.content)

        username: str = data.get("username")
        password: str = data.get("password")

        async with httpx.AsyncClient(http2=True) as client:
            resp = await client.get(
                url="https://passport2-api.chaoxing.com/v11/loginregister",
                params={
                    "cx_xxt_passport": "json",
                    "roleSelect": True,
                    "loginType": 1,
                    "uname": username,
                    "code": password,
                },
            )
            res: dict = resp.json()
            if not res.get("status", False):
                return Response(
                    status_code=401,
                    content=resp.text,
                    headers={"Content-Type": "application/json"},
                )

            resp = await client.get(
                url="https://sso.chaoxing.com/apis/login/userLogin4Uname.do",
            )
            res: dict = resp.json()
            if not res.get("result", False):
                return Response(
                    status_code=401,
                    content=resp.text,
                    headers={"Content-Type": "application/json"},
                )

            return Response(
                status_code=200,
                content=json.dumps(
                    res["msg"]["accountInfo"]["imAccount"],
                    ensure_ascii=False,
                    indent=4,
                ),
                headers={"Content-Type": "application/json"},
            )

    def payload_to_event(self, payload: dict) -> Type[Event]:
        """将平台数据转换为 Event 对象"""
        for cls in EVENT_CLASSES + [Event]:
            try:
                if PYDANTIC_V2:
                    event = cls.model_validate(payload)
                else:
                    event = cls.validate(payload)
                return event
            except ValidationError:
                pass
        raise UnkonwnEventError(payload)
